/*

  JFwadmin v 0.71

  Copyright (C) 1999-2000 Mickael Feledyn
  		     mickael.feledyn@cybergal.com

  This application is free software; you can redistribute it and/or
  modify it under the terms of the GNU Genereal Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  This application is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

*/

package fwadmin;

import javax.swing.table.*;
import java.awt.Color;
import javax.swing.*;
import java.awt.event.*;

public class DefaultPolicy implements ActionListener
{
  private static String defaultInput;
  private static String defaultOutput;
  private static String defaultForward;
  protected static boolean defaultInputNotApplied;
  protected static boolean defaultOutputNotApplied;
  protected static boolean defaultForwardNotApplied;

  protected static final String firewallDenyPolicy = "DENY";
  protected static final String firewallAcceptPolicy = "ACCEPT";
  protected static final String firewallForwardPolicy = "FORWARD";
  protected static final String firewallMasqPolicy = "MASQ";

  private static DefaultPolicy m_instance;

  private static JMenuItem inputDefaultAcceptMenuItem;
  private static JMenuItem inputDefaultDenyMenuItem;
  private static JMenuItem outputDefaultAcceptMenuItem;
  private static JMenuItem outputDefaultDenyMenuItem;
  private static JMenuItem forwardDefaultAcceptMenuItem;
  private static JMenuItem forwardDefaultDenyMenuItem;
  private static JMenuItem resetFirewallToAcceptMenuItem;
  private static JMenuItem resetFirewallToDenyMenuItem;


  private DefaultPolicy() {}

  public static DefaultPolicy getInstance()
  {
    if (m_instance == null) m_instance = new DefaultPolicy();
    return m_instance;
  }

	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() instanceof JMenuItem)
    {
      if (e.getSource().equals(inputDefaultAcceptMenuItem))
      {
        setInputDefaultPolicy(firewallAcceptPolicy);
      }
      if (e.getSource().equals(inputDefaultDenyMenuItem))
      {
        setInputDefaultPolicy(firewallDenyPolicy);
      }
      if (e.getSource().equals(outputDefaultAcceptMenuItem))
      {
        setOutputDefaultPolicy(firewallAcceptPolicy);
      }
      if (e.getSource().equals(outputDefaultDenyMenuItem))
      {
        setOutputDefaultPolicy(firewallDenyPolicy);
      }
      if (e.getSource().equals(forwardDefaultAcceptMenuItem))
      {
        setForwardDefaultPolicy(firewallAcceptPolicy);
      }
      if (e.getSource().equals(forwardDefaultDenyMenuItem))
      {
        setForwardDefaultPolicy(firewallDenyPolicy);
      }
      if (e.getSource().equals(resetFirewallToAcceptMenuItem))
      {
        int res = JOptionPane.showConfirmDialog(Fwadmin.getInstance(), "Please confirm Firewall reset to ACCEPT.", "Continue ?", JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE);
        if (res == JOptionPane.CANCEL_OPTION) return;
        setInputDefaultPolicy(firewallAcceptPolicy);
        setOutputDefaultPolicy(firewallAcceptPolicy);
        setForwardDefaultPolicy(firewallAcceptPolicy);
        InputService.reset();
        OutputService.reset();
        InputRule.reset();
        OutputRule.reset();
        ForwardRule.reset();
        InputOutputRule.reset();
      }
      if (e.getSource().equals(resetFirewallToDenyMenuItem))
      {
        int res = JOptionPane.showConfirmDialog(Fwadmin.getInstance(), "Please confirm Firewall reset to DENY. All connections will be lost when you apply !", "Continue ?", JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE);
        if (res == JOptionPane.CANCEL_OPTION) return;
        setInputDefaultPolicy(firewallDenyPolicy);
        setOutputDefaultPolicy(firewallDenyPolicy);
        setForwardDefaultPolicy(firewallDenyPolicy);
        InputService.reset();
        OutputService.reset();
        InputRule.reset();
        OutputRule.reset();
        ForwardRule.reset();
        InputOutputRule.reset();
      }

      Fwadmin.getInstance().refreshPanels();
    }
  }

public void addPoliciesMenu(JMenuBar theMenuBar)
{
  JMenu m_policyJMenu = new JMenu();
  m_policyJMenu.setText("Policies");

  JMenu inputPolicyMenu = new JMenu();
  inputPolicyMenu.setText("Input policy");
  inputPolicyMenu.addActionListener(this);
  inputDefaultAcceptMenuItem = new JMenuItem();
  inputDefaultAcceptMenuItem.setText("Default to ACCEPT");
  inputDefaultAcceptMenuItem.addActionListener(this);
  inputPolicyMenu.add(inputDefaultAcceptMenuItem);
  inputDefaultDenyMenuItem = new JMenuItem();
  inputDefaultDenyMenuItem.setText("Default to DENY");
  inputDefaultDenyMenuItem.addActionListener(this);
  inputPolicyMenu.add(inputDefaultDenyMenuItem);

  JMenu outputPolicyMenu = new JMenu();
  outputPolicyMenu.setText("Output policy");
  outputPolicyMenu.addActionListener(this);
  outputDefaultAcceptMenuItem = new JMenuItem();
  outputDefaultAcceptMenuItem.setText("Default to ACCEPT");
  outputDefaultAcceptMenuItem.addActionListener(this);
  outputPolicyMenu.add(outputDefaultAcceptMenuItem);
  outputDefaultDenyMenuItem = new JMenuItem();
  outputDefaultDenyMenuItem.setText("Default to DENY");
  outputDefaultDenyMenuItem.addActionListener(this);
  outputPolicyMenu.add(outputDefaultDenyMenuItem);

  JMenu forwardPolicyMenu = new JMenu();
  forwardPolicyMenu.setText("Forward policy");
  forwardPolicyMenu.addActionListener(this);
  forwardDefaultAcceptMenuItem = new JMenuItem();
  forwardDefaultAcceptMenuItem.setText("Default to ACCEPT");
  forwardDefaultAcceptMenuItem.addActionListener(this);
  forwardPolicyMenu.add(forwardDefaultAcceptMenuItem);
  forwardDefaultDenyMenuItem = new JMenuItem();
  forwardDefaultDenyMenuItem.setText("Default to DENY");
  forwardDefaultDenyMenuItem.addActionListener(this);
  forwardPolicyMenu.add(forwardDefaultDenyMenuItem);

  m_policyJMenu.add(inputPolicyMenu);
  m_policyJMenu.add(outputPolicyMenu);
  m_policyJMenu.add(forwardPolicyMenu);
  m_policyJMenu.addSeparator();
  resetFirewallToAcceptMenuItem = new JMenuItem();
  resetFirewallToAcceptMenuItem.setText("Reset Firewall to ACCEPT");
  resetFirewallToAcceptMenuItem.addActionListener(this);
  m_policyJMenu.add(resetFirewallToAcceptMenuItem);
  resetFirewallToDenyMenuItem = new JMenuItem();
  resetFirewallToDenyMenuItem.setText("Reset Firewall to DENY");
  resetFirewallToDenyMenuItem.addActionListener(this);
  m_policyJMenu.add(resetFirewallToDenyMenuItem);


  theMenuBar.add(m_policyJMenu);
}

  public static DefaultTableCellRenderer getPolicyRenderer()
  {
    // Show nice (?) colors for policies.
    DefaultTableCellRenderer policyRenderer = new DefaultTableCellRenderer()
    {
	    public void setValue(Object value)
      {
	      if (value instanceof String)
        {
          String sval = (String)value;
          if(sval.equals(DefaultPolicy.firewallDenyPolicy))
          {
	          setBackground(Color.red.brighter());
	          setForeground(Color.black);
          }
          else
          if(sval.equals(DefaultPolicy.firewallAcceptPolicy))
          {
	          setBackground(Color.green.brighter());
	          setForeground(Color.black);
          }
          else
          if(sval.equals(DefaultPolicy.firewallForwardPolicy))
          {
	          setBackground(Color.blue.darker().darker());
	          setForeground(Color.white);
          }
          else
          if(sval.equals(DefaultPolicy.firewallMasqPolicy))
          {
	          setBackground(Color.blue.brighter());
	          setForeground(Color.white);
          }
          else
          {
	          setBackground(Color.gray);
	          setForeground(Color.black);
          }
          setText(sval);
	      }
	    }
    };

    return policyRenderer;
  }

  public static String getInputDefaultPolicy()
  {
    return defaultInput;
  }
  public static String getOutputDefaultPolicy()
  {
    return defaultOutput;
  }
  public static String getForwardDefaultPolicy()
  {
    return defaultForward;
  }
  public static void setInputDefaultPolicy(String p)
  {
    if (defaultInput != null) defaultInputNotApplied = ! defaultInput.equals(p);
    defaultInput = p;
  }
  public static void setOutputDefaultPolicy(String p)
  {
    if (defaultOutput != null) defaultOutputNotApplied = ! defaultOutput.equals(p);
    defaultOutput = p;
  }
  public static void setForwardDefaultPolicy(String p)
  {
    if (defaultForward != null) defaultForwardNotApplied = ! defaultForward.equals(p);
    defaultForward = p;
  }
}