/*

  JFwadmin v 0.71

  Copyright (C) 1999-2000 Mickael Feledyn
  		     mickael.feledyn@cybergal.com

  This application is free software; you can redistribute it and/or
  modify it under the terms of the GNU Genereal Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  This application is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

*/

package fwadmin;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.event.*;

public class ForwardServicesPanel extends JPanel implements UpdateWindow, ActionListener
{
  String[] columnsNames = {"Forward services / port(s)", "Interface", "Protocol", "Source address", "Source mask", "Policy", "Comment"};
  Object[][] data = {};
  Object[][] old_data = {};
  JTable m_tableView = null;
  JScrollPane m_scrollPane = null;

  JLabel forwardJLabel = new JLabel();


  JLabel      rowHeightLabel;
  JSlider     rowHeightSlider;

  protected ForwardRule selectedRule = null;

  public ForwardServicesPanel()
  {
    this.setLayout(new BorderLayout());
    JPanel topJPanel = new JPanel();
    topJPanel.setLayout(new BorderLayout());
    this.add("North", topJPanel);
    JPanel sliderJPanel = new JPanel();
    sliderJPanel.setLayout(new FlowLayout());
    topJPanel.add("East", sliderJPanel);
    topJPanel.add("West", forwardJLabel);

    rowHeightLabel = new JLabel("Row height:");
	  sliderJPanel.add(rowHeightLabel);

    JPanel bottomJPanel = new JPanel();
    bottomJPanel.setLayout(new FlowLayout());
    JButton removeJButton = new JButton("Remove service");
    removeJButton.addActionListener(this);
    bottomJPanel.add(removeJButton);
    this.add("South", bottomJPanel);

    rowHeightSlider = new JSlider(JSlider.HORIZONTAL, 5, 100, InputServicesPanel.m_rowHeightInitialValue);
	  sliderJPanel.add(rowHeightSlider);
    rowHeightSlider.addChangeListener(new ChangeListener() {
	    public void stateChanged(ChangeEvent e) {
	        int height = ((JSlider)e.getSource()).getValue();
                m_tableView.setRowHeight(height);
                m_tableView.repaint();
	    }
    });

    doDisplayTable();


    MouseAdapter forwardServicesPanelMouseListener = new MouseAdapter() {
      public void mouseClicked(MouseEvent e)
      {
        selectedRule = (ForwardRule)data[m_tableView.getSelectedRow()][columnsNames.length];
      }
    };
    m_tableView.addMouseListener(forwardServicesPanelMouseListener);

  }

  public void actionPerformed(ActionEvent ae)
  {
    if (selectedRule == null)
    {
      JOptionPane.showMessageDialog(this, "No service selected.", "No selection", JOptionPane.ERROR_MESSAGE);
      return;
    }

    ForwardRule.removeForwardRule(selectedRule);
    selectedRule = null;
    Fwadmin.getInstance().refreshPanels();
  }

  public void updateWindow()
  {
    forwardJLabel.setText(" Forward default policy : " + DefaultPolicy.getForwardDefaultPolicy() + ((DefaultPolicy.defaultForwardNotApplied) ? " - NOT APPLIED !" : ""));
    /*if (! DefaultPolicy.getForwardDefaultPolicy().equals(DefaultPolicy.firewallDenyPolicy))
    {
      JOptionPane.showMessageDialog(this, "Your default Forward policy is not " + DefaultPolicy.firewallDenyPolicy, "Warning Warning Warning ", JOptionPane.WARNING_MESSAGE);
    }*/

    data = new Object[ForwardRule.forwardRulesVector.size()][columnsNames.length + 1];
	  for (int i=0; i < ForwardRule.forwardRulesVector.size(); i++)
	  {
		  ForwardRule fr = (ForwardRule) ForwardRule.forwardRulesVector.elementAt(i);
      data[i][0] = fr.getDestinationPortsStringForDisplay();
      data[i][1] = fr.m_interfaceName;
      data[i][2] = fr.getProtocolString();
      data[i][3] = fr.m_sourceAddress.toString();
      data[i][4] = fr.m_sourceMask.toString();
      data[i][5] = fr.m_policy;
      data[i][columnsNames.length] = fr;
      String Comment = "";
      if ((fr.m_flags & Linux22.FLAG_LOGGING) > 0)
      {
        Comment += "logged";
      }
      if (! fr.m_isApplied)
      {
        Comment += "not applied";
      }
      data[i][6] = Comment;
    }

    // Avoid table blinking : verify data changed, and if not, don't update table.
    boolean differenceEncountered = false;
    if (data.length != old_data.length) differenceEncountered = true;
    if (! differenceEncountered)
    {
      for(int i=0; i < data.length; i++)
      {
        if (data[i].length != old_data[i].length) differenceEncountered = true;
        if (differenceEncountered) break;

        for (int j=0; j < data[i].length; j++)
        {
          if (!(data[i][j].equals(old_data[i][j]))) differenceEncountered = true;
          if (differenceEncountered) break;
        }
      }
    }
    // No difference between new and old data, so nothing to do and return.
    if (! differenceEncountered) return;

    old_data = data;

    // Create a model of the data.
    TableModel dataModel = new AbstractTableModel() {
    public int getColumnCount() { return columnsNames.length; }
    public int getRowCount() { return data.length;}
    public Object getValueAt(int row, int col) {return data[row][col];}
    public String getColumnName(int column) {return columnsNames[column];}
    public Class getColumnClass(int c) {return getValueAt(0, c).getClass();}
    public boolean isCellEditable(int row, int col) {return false;}
    public void setValueAt(Object aValue, int row, int column) { data[row][column] = aValue; }
    };


    m_tableView.setModel(dataModel);
    m_tableView.getColumn("Policy").setCellRenderer(DefaultPolicy.getPolicyRenderer());
    m_tableView.sizeColumnsToFit(-1);

  }

  public void doDisplayTable()
  {
    // Create a model of the data.
    TableModel dataModel = new AbstractTableModel() {
    public int getColumnCount() { return columnsNames.length; }
    public int getRowCount() { return data.length;}
    public Object getValueAt(int row, int col) {return data[row][col];}
    public String getColumnName(int column) {return columnsNames[column];}
    public Class getColumnClass(int c) {return getValueAt(0, c).getClass();}
    public boolean isCellEditable(int row, int col) {return false;}
    public void setValueAt(Object aValue, int row, int column) { data[row][column] = aValue; }
    };

    // Create the table
    m_tableView = new JTable(dataModel);
    m_tableView.setRowHeight(InputServicesPanel.m_rowHeightInitialValue);
    m_scrollPane = new JScrollPane(m_tableView);
    this.add("Center", m_scrollPane);
  }
}
