/*

  JFwadmin v 0.71

  Copyright (C) 1999-2000 Mickael Feledyn
  		     mickael.feledyn@cybergal.com

  This application is free software; you can redistribute it and/or
  modify it under the terms of the GNU Genereal Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  This application is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

*/

package fwadmin;

import java.util.*;
import java.io.*;

public class Linux22
{
protected static Hashtable port2serviceHashtable;
protected static Hashtable service2portHashtable;
protected static Vector servicesVector;
protected static Hashtable m_varsHashtable;


protected static PrintWriter m_printWriter;

public static int getProtocolConstant(String prot)
{
  if(prot.equals("tcp")) return Linux22.IPPROTO_TCP;
  if(prot.equals("udp")) return Linux22.IPPROTO_UDP;
  if(prot.equals("icmp")) return Linux22.IPPROTO_ICMP;
  if(prot.equals("all")) return Linux22.IPPROTO_IP;
  return 0;
}

public static void applyRules() throws Exception
{
  applyInputRules();
  applyOutputRules();
  applyForwardRules();
}

public static void saveBootScript(PrintWriter pw) throws Exception
{
  Linux22.m_printWriter = pw;
  writeScriptInitVars(pw);
  Linux22.applyRules();
  pw.flush();
  pw.close();
  Linux22.m_printWriter = null;
}

public static void setupScriptVar(String varname, String varvalue, PrintWriter pw)
{
  varname = varname.toUpperCase();
  m_varsHashtable.put(varvalue, varname);
  m_printWriter.println(varname + "=" + varvalue);
}

public static void writeScriptInitVars(PrintWriter pw)
{
  m_printWriter.println("#!/bin/sh");
  m_printWriter.println("# Firewall Script created " + (new Date()).toString());
  m_printWriter.println("# Script generator : " + Fwadmin.getNameAndVersion());
  m_printWriter.println();
  m_printWriter.println("# Set up vars");

  m_varsHashtable = new Hashtable();

  for (Enumeration e = IPInterface.interfacesHashtable.elements(); e.hasMoreElements(); )
  {
    IPInterface ipi = (IPInterface)e.nextElement();
    setupScriptVar(ipi.getName() + "_IF", ipi.getName(), pw);

    // Automatic determination of IP address within the script
    m_printWriter.println(ipi.getName().toUpperCase() + "_IP=`/sbin/ifconfig " + ipi.getName() + "|sed -n -e \"s/^[     ]*inet addr\\:\\([0-9\\.]*\\).*$/\\1/p\"`");
    m_varsHashtable.put(ipi.getAddress().toString(), (ipi.getName() + "_IP").toUpperCase());

    String varvalue = (new IPAddress(ipi.getAddress().getLong() & ipi.getMask().getLong())).toString();
    varvalue += "/" + (new IPMask(ipi.getMask().getLong())).toString();
    setupScriptVar(ipi.getName() + "_NET", varvalue, pw);
  }

  // Remove "anywhere" networks
  String anywhereNet = "0.0.0.0/0.0.0.0";
  while(true)
  {
    Object o = m_varsHashtable.get(anywhereNet);
    if (o != null)
      m_varsHashtable.remove(anywhereNet);
    else
      break;
  }
  setupScriptVar("ANYWHERE", anywhereNet, pw);
  setupScriptVar("PORTS", "0:65535", pw);

  m_printWriter.println();
  m_printWriter.println("# Here are the rules");
}
public static void systemExec(String [] args) throws Exception
{
  if (m_printWriter != null)
  {
    for (int i=0; i < args.length; i++)
    {
      String toPrint = args[i];
      if (m_varsHashtable.get(toPrint) != null) toPrint = "$" + m_varsHashtable.get(toPrint);
      m_printWriter.print(toPrint + ((i<args.length-1) ? " " : ""));
    }
    m_printWriter.println();
    return;
  }

  Process p = Runtime.getRuntime().exec(args);
  p.waitFor();
  int res = p.exitValue();
  if (res != 0) throw new Exception("There was an error.");
}

public static void applyInputRules() throws Exception
{
    // Flush input rules
    systemExec(new String[] {"/sbin/ipchains", "-F", "input"});
    systemExec(new String[] {"/sbin/ipchains", "-P", "input", DefaultPolicy.getInputDefaultPolicy()});
    // Now execute commands to establish rules
    Vector sortedInputRules = sortRules(InputRule.inputRulesVector);
    for (int i=0; i < sortedInputRules.size(); i++)
    {
      InputOutputRule ior = (InputOutputRule)sortedInputRules.elementAt(i);
      Vector v_args = new Vector();
      v_args.addElement("/sbin/ipchains");
      v_args.addElement("-A");
      v_args.addElement("input");
      v_args.addElement("-p");
      v_args.addElement(ior.getProtocolString());
      v_args.addElement("-j");
      v_args.addElement(ior.m_policy);
      v_args.addElement("-i");
      v_args.addElement(ior.m_interfaceName);
      v_args.addElement("-s");
      v_args.addElement(ior.m_sourceAddress.toString() + ior.m_sourceMask.getMaskStringForScript());
      String s_ports = ior.getSourcePortsString();
      if (! s_ports.equals("all")) v_args.addElement(s_ports);
      v_args.addElement("-d");
      v_args.addElement(ior.m_destinationAddress.toString() + ior.m_destinationMask.getMaskStringForScript());
      String d_ports = ior.getDestinationPortsString();
      if (! d_ports.equals("all")) v_args.addElement(d_ports);
      if ((ior.m_flags & Linux22.FLAG_LOGGING) > 0) v_args.addElement("-l");

      String [] args = new String[v_args.size()];
      for (int j=0; j < v_args.size(); j++)
        args[j] = (String)v_args.elementAt(j);

      systemExec(args);
    }
}

public static void applyOutputRules() throws Exception
{
    // Flush output rules
    systemExec(new String[] {"/sbin/ipchains", "-F", "output"});
    systemExec(new String[] {"/sbin/ipchains", "-P", "output", DefaultPolicy.getOutputDefaultPolicy()});
    // Now execute commands to establish rules
    Vector sortedOutputRules = sortRules(OutputRule.outputRulesVector);
    for (int i=0; i < sortedOutputRules.size(); i++)
    {
      InputOutputRule ior = (InputOutputRule)sortedOutputRules.elementAt(i);
      Vector v_args = new Vector();
      v_args.addElement("/sbin/ipchains");
      v_args.addElement("-A");
      v_args.addElement("output");
      v_args.addElement("-p");
      v_args.addElement(ior.getProtocolString());
      v_args.addElement("-j");
      v_args.addElement(ior.m_policy);
      v_args.addElement("-i");
      v_args.addElement(ior.m_interfaceName);
      v_args.addElement("-s");
      v_args.addElement(ior.m_sourceAddress.toString() + ior.m_sourceMask.getMaskStringForScript());
      String s_ports = ior.getSourcePortsString();
      if (! s_ports.equals("all")) v_args.addElement(s_ports);
      v_args.addElement("-d");
      v_args.addElement(ior.m_destinationAddress.toString() + ior.m_destinationMask.getMaskStringForScript());
      String d_ports = ior.getDestinationPortsString();
      if (! d_ports.equals("all")) v_args.addElement(d_ports);
      if ((ior.m_flags & Linux22.FLAG_LOGGING) > 0) v_args.addElement("-l");

      String [] args = new String[v_args.size()];
      for (int j=0; j < v_args.size(); j++)
        args[j] = (String)v_args.elementAt(j);

      systemExec(args);
    }
}

public static void applyForwardRules() throws Exception
{
    // Flush output rules
    systemExec(new String[] {"/sbin/ipchains", "-F", "forward"});
    systemExec(new String[] {"/sbin/ipchains", "-P", "forward", DefaultPolicy.getForwardDefaultPolicy()});
    // Now execute commands to establish rules
    Vector sortedForwardRules = sortRules(ForwardRule.forwardRulesVector);
    for (int i=0; i < sortedForwardRules.size(); i++)
    {
      ForwardRule fr = (ForwardRule)sortedForwardRules.elementAt(i);
      Vector v_args = new Vector();
      v_args.addElement("/sbin/ipchains");
      v_args.addElement("-A");
      v_args.addElement("forward");
      v_args.addElement("-p");
      v_args.addElement(fr.getProtocolString());
      v_args.addElement("-j");
      v_args.addElement(fr.m_policy);
      v_args.addElement("-i");
      v_args.addElement(fr.m_interfaceName);
      v_args.addElement("-s");
      v_args.addElement(fr.m_sourceAddress.toString() + fr.m_sourceMask.getMaskStringForScript());
      String s_ports = fr.getSourcePortsString();
      if (! s_ports.equals("all")) v_args.addElement(s_ports);
      v_args.addElement("-d");
      v_args.addElement(fr.m_destinationAddress.toString() + fr.m_destinationMask.getMaskStringForScript());
      String d_ports = fr.getDestinationPortsString();
      if (! d_ports.equals("all")) v_args.addElement(d_ports);
      if ((fr.m_flags & Linux22.FLAG_LOGGING) > 0) v_args.addElement("-l");

      String [] args = new String[v_args.size()];
      for (int j=0; j < v_args.size(); j++)
        args[j] = (String)v_args.elementAt(j);

      systemExec(args);
    }
}

public static Vector sortRules(Vector v)
{
  Vector res = new Vector();
  for (int i=0; i < v.size(); i++)
  {
    InputOutputRule ior = (InputOutputRule)v.elementAt(i);
    if (res.size() == 0)
      res.addElement(ior);
    else
    if (ior.compareTo((InputOutputRule)res.elementAt(res.size()-1)) >= 0)
      res.addElement(ior);
    else
    {
      for (int j=0; j < res.size(); j++)
      {
        if (ior.compareTo((InputOutputRule)res.elementAt(j)) < 0)
        {
          res.insertElementAt(ior, j);
          break;
        }
      }
    }
  }
  return res;
}

public static void readNetConfig()
{
  // Read interfaces.
  try
  {
    Process p = Runtime.getRuntime().exec(new String[] {"/sbin/ifconfig"});
    BufferedReader bf = new BufferedReader(new InputStreamReader(p.getInputStream()));
    //FileReader fr_route = new FileReader("C:\\proc\\net\\ifconfig");
    //BufferedReader bf = new BufferedReader(fr_route);
    int lineNumber = 0;
    int lineNumberBeginInterfaceDefinition = 0;
    String if_name = null;
    while(true)
    {
      String line = bf.readLine(); lineNumber++;
      if(line == null) break;
      //System.out.println("read : " + line);
      if ((! line.startsWith(" ")) && (! line.startsWith("\t")))
      {
        lineNumberBeginInterfaceDefinition = lineNumber;
        StringTokenizer st = new StringTokenizer(line);
        if (st.hasMoreTokens()) if_name = st.nextToken();
      }
      if (lineNumber == (lineNumberBeginInterfaceDefinition + 1))
      {
        StringTokenizer st = new StringTokenizer(line, " :");
        String if_addr = null;
        String if_mask = null;
        while(st.hasMoreTokens())
        {
          String t1 = st.nextToken();
          if (t1.equals("addr")) if_addr = st.nextToken();
          if (t1.equals("Mask")) if_mask = st.nextToken();
          if (if_mask != null) new IPInterface(if_name, if_addr, if_mask);
        }
      }
    }
  }
  catch(Exception e)
  {
    System.out.println(e.toString());
  }

  // Now read routing.
  try
  {
    FileReader fr_route = new FileReader("/proc/net/route");
    //FileReader fr_route = new FileReader("C:\\proc\\net\\route");
    BufferedReader bf = new BufferedReader(fr_route);
    int lineNumber = 0;
    int lineNumberBeginInterfaceDefinition = 0;
    while(true)
    {
      String line = bf.readLine(); lineNumber++;
      if(line == null) break;
      if (lineNumber > 1)
      {
        StringTokenizer st = new StringTokenizer(line, " \t");
          String if_name = st.nextToken();
          String destination = st.nextToken();
          String gateway = st.nextToken();
          st.nextToken();
          st.nextToken();
          st.nextToken();
          st.nextToken();
          String genmask = st.nextToken();
          IPInterface.getInterface(if_name).addRoute(destination, gateway, genmask);
      }
    }
  }
  catch(Exception e)
  {
    System.out.println(e.toString());
    e.printStackTrace();
  }

  initServicesHashtables();
}

public static void readFirewallRules()
{
  // Now read chains default policy
	try
	{
    FileReader fr_policy = new FileReader("/proc/net/ip_fwnames");
    //FileReader fr_policy = new FileReader("C:\\proc\\net\\ip_fwnames");
    BufferedReader bf = new BufferedReader(fr_policy);
	  while(true)
	  {
		  String line = bf.readLine();
      if(line == null) break;

		  if (line.compareTo("") > 0)
		  {
			  StringTokenizer st = new StringTokenizer(line);
			  String chain = st.nextToken();
			  if (chain.equals("input"))
			  {
				  DefaultPolicy.setInputDefaultPolicy(st.nextToken());
          DefaultPolicy.defaultInputNotApplied = false;
  			}
			  if (chain.equals("output"))
			  {
				  DefaultPolicy.setOutputDefaultPolicy(st.nextToken());
          DefaultPolicy.defaultOutputNotApplied = false;
  			}
			  if (chain.equals("forward"))
			  {
				  DefaultPolicy.setForwardDefaultPolicy(st.nextToken());
          DefaultPolicy.defaultForwardNotApplied = false;
  			}
      }
    }
	}
	catch(java.io.FileNotFoundException fnfe)
	{
		System.out.println("can't open file " + fnfe.toString());
	}
	catch(java.io.IOException ioe)
	{
		System.out.println(ioe.toString());
	}

  // Now read firewall rules
	try
	{
    FileReader fr_route = new FileReader("/proc/net/ip_fwchains");
    //FileReader fr_route = new FileReader("C:\\proc\\net\\ip_fwchains");
    BufferedReader bf = new BufferedReader(fr_route);

	  int lineNumber = 0;
    int inputRulesCount = 0;
    int outputRulesCount = 0;
    int forwardRulesCount = 0;
	  while(true)
	  {
		  String rawRule = bf.readLine();
      if(rawRule == null) break;

		  if (rawRule.compareTo("") > 0)
		  {
			  StringTokenizer st = new StringTokenizer(rawRule);
			  String ruleType = st.nextToken();
			  if (ruleType.equals("input"))
			  {
				  InputRule r = new InputRule(rawRule);
          r.m_isApplied = true;
				  InputRule.inputRulesVector.addElement(r);
  			}
	  		else
		  	if (ruleType.equals("output"))
			  {
  				OutputRule r = new OutputRule(outputRulesCount++, rawRule);
          r.m_isApplied = true;
	  			OutputRule.outputRulesVector.addElement(r);
		  	}
			  else
  			if (ruleType.equals("forward"))
	  		{
		  		ForwardRule r = new ForwardRule(rawRule);
          r.m_isApplied = true;
			  	ForwardRule.forwardRulesVector.addElement(r);
	  		}
  			else
		  	{
          // tbd
			  }

  		}
		  lineNumber++;
    }
	}
	catch(java.io.FileNotFoundException fnfe)
	{
		System.out.println("can't open file " + fnfe.toString());
	}
	catch(java.io.IOException ioe)
	{
		System.out.println(ioe.toString());
	}
}

public static String getServiceName(int port, String protocolType)
{
  String theKey = "" + port + "/" + protocolType;
  String theElement = (String)port2serviceHashtable.get(theKey);
  if (theElement != null) return theElement;

  return "" + port;
}

public static int getServicePortNumber(String name, String protocolType)
{
  try
  {
    int i_port = Integer.parseInt(name);
    return i_port;
  }
  catch(NumberFormatException nfe)
  {}
  String theKey = (new StringTokenizer(name)).nextToken() + "/" + protocolType;
  Integer theInteger = (Integer)service2portHashtable.get(theKey);
  if (theInteger == null) return 0;
  return theInteger.intValue();
}

private static void initServicesHashtables()
{
  port2serviceHashtable = new Hashtable();
  service2portHashtable = new Hashtable();
  servicesVector = new Vector();

	try
	{
    FileReader fr_services = new FileReader("/etc/services");
    //FileReader fr_services = new FileReader("C:\\proc\\net\\services");
    BufferedReader bf = new BufferedReader(fr_services);
	  while(true)
	  {
		  String line = bf.readLine();
      if(line == null) break;

      StringTokenizer st = new StringTokenizer(line, " \t\r\n");
      while (st.hasMoreElements())
      {
        String name = st.nextToken();
        if (name.startsWith("#")) break; // Comment
        String port = st.nextToken();
        port2serviceHashtable.put(port, name);
        StringTokenizer st2 = new StringTokenizer(port, "/");
        Integer portNumber = new Integer(st2.nextToken());
        String prot = st2.nextToken();
        service2portHashtable.put(name + "/" + prot, portNumber);
        boolean inserted = false;
        name += " (" + portNumber + ")";
        for (int i=0; i < servicesVector.size(); i++)
        {
            int cmp = name.compareTo((String)servicesVector.elementAt(i));
            if(cmp == 0)
            {
              // Service name already present
              inserted = true;
              break;
            }
            if(cmp < 0)
            {
              servicesVector.insertElementAt(name, i);
              inserted = true;
              break;
            }
        }
        if (! inserted) servicesVector.addElement(name);
        break;
      }
    }
	}
	catch(java.io.FileNotFoundException fnfe)
	{
		System.out.println("can't open file " + fnfe.toString());
	}
	catch(java.io.IOException ioe)
	{
		System.out.println(ioe.toString());
	}
}

	static final int FLAG_LOGGING = 1;	   /* This flag indicates logging.  */

	static final int IPPROTO_IP = 0;	   /* Dummy protocol for TCP.  */
	static final int IPPROTO_HOPOPTS = 0;   /* IPv6 Hop-by-Hop options.  */
	static final int IPPROTO_ICMP = 1;	   /* Internet Control Message Protocol.  */
	static final int IPPROTO_IGMP = 2;	   /* Internet Group Management Protocol. */
	static final int IPPROTO_IPIP = 4;	   /* IPIP tunnels (older KA9Q tunnels use 94).  */
	static final int IPPROTO_TCP = 6;	   /* Transmission Control Protocol.  */
	static final int IPPROTO_EGP = 8;	   /* Exterior Gateway Protocol.  */
	static final int IPPROTO_PUP = 12;	   /* PUP protocol.  */
	static final int IPPROTO_UDP = 17;	   /* User Datagram Protocol.  */
	static final int IPPROTO_IDP = 22;	   /* XNS IDP protocol.  */
	static final int IPPROTO_TP = 29;	   /* SO Transport Protocol Class 4.  */
	static final int IPPROTO_IPV6 = 41;     /* IPv6 header.  */
	static final int IPPROTO_ROUTING = 43;  /* IPv6 routing header.  */
	static final int IPPROTO_FRAGMENT = 44; /* IPv6 fragmentation header.  */
	static final int IPPROTO_RSVP = 46;	   /* Reservation Protocol.  */
	static final int IPPROTO_GRE = 47;	   /* General Routing Encapsulation.  */
	static final int IPPROTO_ESP = 50;      /* encapsulating security payload.  */
	static final int IPPROTO_AH = 51;       /* authentication header.  */
	static final int IPPROTO_ICMPV6 = 58;   /* ICMPv6.  */
	static final int IPPROTO_NONE = 59;     /* IPv6 no next header.  */
	static final int IPPROTO_DSTOPTS = 60;  /* IPv6 destination options.  */
	static final int IPPROTO_MTP = 92;	   /* Multicast Transport Protocol.  */
	static final int IPPROTO_ENCAP = 98;	   /* Encapsulation Header.  */
	static final int IPPROTO_PIM = 103;	   /* Protocol Independent Multicast.  */
	static final int IPPROTO_RAW = 255;	   /* Raw IP packets.  */

	static final int IP_FW_MAX_PORTS = 10;  /* A reasonable maximum, like said in ip_fw.h */
	static final int IP_FW_F_ALL = 0x0000;	/* This is a universal packet firewall*/
	static final int IP_FW_F_TCP = 0x0001;	/* This is a TCP packet firewall      */
	static final int IP_FW_F_UDP = 0x0002;	/* This is a UDP packet firewall      */
	static final int IP_FW_F_ICMP = 0x0003;	/* This is a ICMP packet firewall     */
	static final int IP_FW_F_KIND = 0x0003;	/* Mask to isolate firewall kind      */
	static final int IP_FW_F_ACCEPT = 0x0004;	/* This is an accept firewall (as *
				 *         opposed to a deny firewall)*
				 *                                    */
	static final int IP_FW_F_SRNG = 0x0008;	/* The first two src ports are a min  *
				 * and max range (stored in host byte *
				 * order).                            *
				 *                                    */
	static final int IP_FW_F_DRNG = 0x0010;	/* The first two dst ports are a min  *
				 * and max range (stored in host byte *
				 * order).                            *
				 * (ports[0] <= port <= ports[1])     *
				 *                                    */
	static final int IP_FW_F_PRN = 0x0020;		/* In verbose mode print this firewall*/
	static final int IP_FW_F_BIDIR = 0x0040;	/* For bidirectional firewalls        */
	static final int IP_FW_F_TCPSYN = 0x0080;	/* For tcp packets-check SYN only     */
	static final int IP_FW_F_ICMPRPL = 0x0100;	/* Send back icmp unreachable packet  */
	static final int IP_FW_F_MASQ = 0x0200;		/* Masquerading			      */
	static final int IP_FW_F_TCPACK = 0x0400;	/* For tcp-packets match if ACK is set*/
	static final int IP_FW_F_REDIR = 0x0800;	/* Redirect to local port fw_pts[n]   */
	static final int IP_FW_F_ACCTIN = 0x1000;	/* Account incoming packets only.     */
	static final int IP_FW_F_ACCTOUT = 0x2000;	/* Account outgoing packets only.     */

	static final int IP_FW_F_MASK = 0x3FFF;		/* All possible flag bits mask        */	
/**
 * LinuxIPFWConstants constructor comment.
 */
private Linux22() {
	super();
}

}