/*

  JFwadmin v 0.71

  Copyright (C) 1999-2000 Mickael Feledyn
  		     mickael.feledyn@cybergal.com

  This application is free software; you can redistribute it and/or
  modify it under the terms of the GNU Genereal Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.

  This application is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

*/

package fwadmin;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.event.*;

public class MiscRulesPanel extends JPanel implements UpdateWindow
{
  String[] columnsNames = {"Interface", "Protocol", "Source port(s)", "Source address", "Source mask", "Destination port(s)", "Destination address", "Destination mask", "Policy", "Comment"};
  Object[][] data = {};
  Object[][] old_data = {};
  JTable m_tableView = null;
  JScrollPane m_scrollPane = null;

  JLabel miscJLabel = new JLabel();


  JLabel      rowHeightLabel;
  JSlider     rowHeightSlider;

  public MiscRulesPanel()
  {
    this.setLayout(new BorderLayout());
    JPanel topJPanel = new JPanel();
    topJPanel.setLayout(new BorderLayout());
    this.add("North", topJPanel);
    JPanel sliderJPanel = new JPanel();
    sliderJPanel.setLayout(new FlowLayout());
    topJPanel.add("East", sliderJPanel);
    topJPanel.add("West", miscJLabel);

    rowHeightLabel = new JLabel("Row height:");
	  sliderJPanel.add(rowHeightLabel);

    JPanel bottomJPanel = new JPanel();
    bottomJPanel.setLayout(new FlowLayout());
    JLabel bogusJLabel = new JLabel("You should not have misc rules (unless you're an advanced user :) )");
    bottomJPanel.add(bogusJLabel);
    this.add("South", bottomJPanel);

    rowHeightSlider = new JSlider(JSlider.HORIZONTAL, 5, 100, InputServicesPanel.m_rowHeightInitialValue);
	  sliderJPanel.add(rowHeightSlider);
    rowHeightSlider.addChangeListener(new ChangeListener() {
	    public void stateChanged(ChangeEvent e) {
	        int height = ((JSlider)e.getSource()).getValue();
                m_tableView.setRowHeight(height);
                m_tableView.repaint();
	    }
    });

    doDisplayTable();
  }

  public void updateWindow()
  {
    miscJLabel.setText(" Miscellaneous Rules");

    data = new Object[InputOutputRule.miscRulesVector.size()][columnsNames.length];
	  for (int i=0; i < InputOutputRule.miscRulesVector.size(); i++)
	  {
		  InputOutputRule ior = (InputOutputRule) InputOutputRule.miscRulesVector.elementAt(i);

      data[i][0] = ior.m_interfaceName;
      data[i][1] = ior.getProtocolString();
      data[i][2] = ior.getSourcePortsStringForDisplay();
      data[i][3] = ior.m_sourceAddress.toString();
      data[i][4] = ior.m_sourceMask.toString();
      data[i][5] = ior.getDestinationPortsStringForDisplay();
      data[i][6] = ior.m_destinationAddress.toString();
      data[i][7] = ior.m_destinationMask.toString();
      data[i][8] = ior.m_policy;
      String Comment = "";
      if ((ior.m_flags & Linux22.FLAG_LOGGING) > 0)
      {
        Comment += "logged";
      }
      data[i][9] = Comment;
    }

    // Avoid table blinking : verify data changed, and if not, don't update table.
    boolean differenceEncountered = false;
    if (data.length != old_data.length) differenceEncountered = true;
    if (! differenceEncountered)
    {
      for(int i=0; i < data.length; i++)
      {
        if (data[i].length != old_data[i].length) differenceEncountered = true;
        if (differenceEncountered) break;

        for (int j=0; j < data[i].length; j++)
        {
          if (!(data[i][j].equals(old_data[i][j]))) differenceEncountered = true;
          if (differenceEncountered) break;
        }
      }
    }
    // No difference between new and old data, so nothing to do and return.
    if (! differenceEncountered) return;

    old_data = data;

    // Create a model of the data.
    TableModel dataModel = new AbstractTableModel() {
    public int getColumnCount() { return columnsNames.length; }
    public int getRowCount() { return data.length;}
    public Object getValueAt(int row, int col) {return data[row][col];}
    public String getColumnName(int column) {return columnsNames[column];}
    public Class getColumnClass(int c) {return getValueAt(0, c).getClass();}
    public boolean isCellEditable(int row, int col) {return false;}
    public void setValueAt(Object aValue, int row, int column) { data[row][column] = aValue; }
    };


    m_tableView.setModel(dataModel);
    m_tableView.getColumn("Policy").setCellRenderer(DefaultPolicy.getPolicyRenderer());
    m_tableView.sizeColumnsToFit(-1);

  }

  public void doDisplayTable()
  {
    // Create a model of the data.
    TableModel dataModel = new AbstractTableModel() {
    public int getColumnCount() { return columnsNames.length; }
    public int getRowCount() { return data.length;}
    public Object getValueAt(int row, int col) {return data[row][col];}
    public String getColumnName(int column) {return columnsNames[column];}
    public Class getColumnClass(int c) {return getValueAt(0, c).getClass();}
    public boolean isCellEditable(int row, int col) {return false;}
    public void setValueAt(Object aValue, int row, int column) { data[row][column] = aValue; }
    };

    // Create the table
    m_tableView = new JTable(dataModel);
    m_tableView.setRowHeight(InputServicesPanel.m_rowHeightInitialValue);
    m_scrollPane = new JScrollPane(m_tableView);
    this.add("Center", m_scrollPane);
  }
}